/////////////////////////////////
// ramen_timer - ラーメン・タイマー AquesTalk pico LSIを使用
//   
// by N.Yamazaki AQUEST Corp.  <http://www.a-quest.com>


#include <Wire.h>	// I2C通信用ライブラリ

const int ledPin = 13;	// LED のピン番号
const int swPin = 2;    // タクトスイッチのピン番号

int sec10;		// 残時間カウンタ 単位は[0.1秒]
int lastState;	// 一つ前のタクトスイッチの状態
int setMinute;	// セット時間　単位は[分]

void setup()
{
	Wire.begin();  // ArduinoをI2C Masterとして初期化

	pinMode(swPin, INPUT_PULLUP);	// スイッチ端子を入力(プルアップあり）に
	pinMode(ledPin, OUTPUT);		// LED端子を出力に

	sec10 = -5;
	lastState=HIGH;

	while(AquesTalk_IsBusy()) ; // Ready待ち
	AquesTalk_Synthe("#J"); // 「ポン！」チャイム音出力
}

void loop()
{
	/////////////////////////////////////
	// タクトスイッチの処理
	/////////////////////////////////////
	int state = digitalRead(swPin);   
	if(lastState==HIGH && state==LOW){ // ボタンが押された
		setMinute = (sec10+650)/600; // 1分追加（65秒追加して分単位に切り捨て）
		sec10 = setMinute*600;	// 0.1秒単位のカウンタ
		String strMsg = GenMinuteMessage(setMinute);
		while(AquesTalk_IsBusy()) ; // Ready待ち
		AquesTalk_Synthe(strMsg); // 「１分」、「２分」、...
	}
	lastState = state; //現在のスイッチ状態を保存


	/////////////////////////////////////
	// カウンタ値に応じた処理
	/////////////////////////////////////
	if(sec10>0) { // カウントダウン中
		sec10--;	// 0になるまでカウントダウン
		LedBrink(); // カウントダウン中はLED点滅する
	}
	else if(sec10==0){ // Time up
		if( !AquesTalk_IsBusy() ){
			String strMsg = GenMinuteMessage(setMinute) + "+ta'ttayo.";
			AquesTalk_Synthe(strMsg);	// 「○分たったよ！」
			sec10--; 
			digitalWrite(ledPin, LOW); // LED OFF
		}
	}
	else if(-5<sec10 && sec10<0){ // Time up 以降 ４回繰り返す
		if( !AquesTalk_IsBusy() ){
			// 「ラーメン、ラーメン、らーめんで・き・た！」
			AquesTalk_Synthe("ra'-menn/ra'-menn/ra,ame-nn-/de',ki,ta'/...");
			sec10--;
		}
	}

	delay(100);  // 0.1sec 待ち
}

/*----------------------------------------------------
	ローカル関数
-----------------------------------------------------*/
// 「○分」の音声記号列生成
//  例) "<NUMK VAL=3 COUNTER=fun>"
String GenMinuteMessage(int min)
{
    String str = "<NUMK VAL=";
    str += String(min, DEC);
    str += " COUNTER=fun>";
    return str;
}

// 10回呼び出しにつき1回、LEDを点灯
void  LedBrink()
{
  static int count=0;
  
  // counter update
  count++;
  if(count==10) count=0;
  
  if(count==0) digitalWrite(ledPin, HIGH);
  else         digitalWrite(ledPin, LOW);
}


/*----------------------------------------------------
	音声合成用の関数
-----------------------------------------------------*/
#define I2C_ADDR_AQUESTALK 0x2E // AquesTalk pico LSIのデフォルトのI2Cアドレス

// LSIがコマンドを受信可能かチェック
// 戻り値　0:Ready 1:Busy 2:Error
int AquesTalk_IsBusy()
{
	delay(10); // Busy応答は10msec以上待つ必要がある 連続して呼ばれた場合のため
	Wire.requestFrom(I2C_ADDR_AQUESTALK, 1);
	if(Wire.available()>0){
		byte c = Wire.read();
		if(c=='>')	return 0;	// Ready応答
		else		return 1;	// busy応答
	}
	else {
		return 2; //ERR: NOACK または応答が無い。I2Cの配線をチェックすべし
	}
}


// 音声合成開始	引数に音声記号列を指定
void AquesTalk_Synthe(String &strMsg)
{
    char msg[256];
    strMsg.toCharArray(msg, 256);
	AquesTalk_Synthe(msg);
}

// 音声合成開始	引数に音声記号列を指定
// 最後に"\r"を送信
void AquesTalk_Synthe(const char *msg)
{
	AquesTalk_Cmd(msg);
	AquesTalk_Cmd("\r");
}

// LSI にコマンド送信
void AquesTalk_Cmd(const char *msg)
{
	// Wireの制約で、一度に送れるのは32byteまで
	// AquesTalk picoへは一度に128byteまで送れるので、
	// Wire.beginTransmission()～Wire.endTransmission()を複数回に分けて呼び出す
	const char *p = msg;
	for(;*p!=0;){
		Wire.beginTransmission(I2C_ADDR_AQUESTALK);
		// Wireの制約で、一度に送れるのは32byteまで
		for(int i=0;i<32;i++){
			Wire.write(*p++);
			if(*p==0) break;
		}
		Wire.endTransmission(); // 実際はこのタイミングで送信される
	}
}

